/*---------------------------------------------------------------------------*\
  =========                 |
  \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox
   \\    /   O peration     |
    \\  /    A nd           | Copyright held by original author
     \\/     M anipulation  |
-------------------------------------------------------------------------------
License
    This file is part of OpenFOAM.

    OpenFOAM is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
    Free Software Foundation; either version 2 of the License, or (at your
    option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM; if not, write to the Free Software Foundation,
    Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

\*---------------------------------------------------------------------------*/

#include "collidingSolidParticleCloud.H"
#include "fvMesh.H"
#include "volFields.H"
#include "interpolationCellPoint.H"
#include "volPointInterpolation.H"
#include "vector.H"

// * * * * * * * * * * * * * * Static Data Members * * * * * * * * * * * * * //

namespace Foam
{
    defineParticleTypeNameAndDebug(collidingSolidParticle, 0);
    defineTemplateTypeNameAndDebug(Cloud<collidingSolidParticle>, 0);
};

// * * * * * * * * * * * * * * * * Constructors  * * * * * * * * * * * * * * //

Foam::collidingSolidParticleCloud::collidingSolidParticleCloud
(
    const fvMesh& mesh,
    const word& cloudName
)
:
    Cloud<collidingSolidParticle>(mesh, cloudName, false),
    mesh_(mesh),
    particleProperties_
    (
        IOobject
        (
            "particleProperties",
            mesh_.time().constant(),
            mesh_,
            IOobject::MUST_READ,
            IOobject::NO_WRITE
        )
    ),
    rhop_(dimensionedScalar(particleProperties_.lookup("rhop")).value()),
    e_(dimensionedScalar(particleProperties_.lookup("e")).value()),
    mu_(dimensionedScalar(particleProperties_.lookup("mu")).value())
{
    collidingSolidParticle::readFields(*this);
}


// * * * * * * * * * * * * * * * Member Functions  * * * * * * * * * * * * * //
void Foam::collidingSolidParticleCloud::checkCell()
{
List<label> lcell((*this).size());
List<scalar> ld((*this).size());
List<vector> lU((*this).size());
List<vector> lposition((*this).size());
bool collision;
label i=0;
forAllConstIter(Cloud<collidingSolidParticle>,*this,iter)
{
const collidingSolidParticle& p=iter();
lcell[i]=p.cell();
Info <<"Particle " <<i<< " is in cell "<<lcell[i]<<endl;
lU[i]=p.U();
ld[i]=p.d();
lposition[i]=p.position();
i++;
}
Info <<"p0-p1 = "<<mag(lposition[0]-lposition[1])<<endl;
Info <<"Diameter = "<<(ld[0]+ld[1])/2<<endl;
//Only works for two particles.
if (lcell[0]==lcell[1] && mag(lposition[0]-lposition[1])<=(ld[0]+ld[1])/2)
{
collision=true;
}
else
{
collision=false;
}
collision_=collision;
U0_=lU[0];
}


void Foam::collidingSolidParticleCloud::move(const dimensionedVector& g)
{
    const volScalarField& rho = mesh_.lookupObject<const volScalarField>("rho");
    const volVectorField& U = mesh_.lookupObject<const volVectorField>("U");
    const volScalarField& nu = mesh_.lookupObject<const volScalarField>("nu");

    pointMesh pMesh(U.mesh());
    volPointInterpolation vpi(U.mesh(), pMesh);

    interpolationCellPoint<scalar> rhoInterp(vpi, rho);
    interpolationCellPoint<vector> UInterp(vpi, U);
    interpolationCellPoint<scalar> nuInterp(vpi, nu);

    collidingSolidParticle::trackData td(*this, rhoInterp, UInterp, nuInterp, g.value());

    Cloud<collidingSolidParticle>::move(td);
}


void Foam::collidingSolidParticleCloud::writeFields() const
{
    collidingSolidParticle::writeFields(*this);
}


// ************************************************************************* //
