/*---------------------------------------------------------------------------*\
  =========                 |
  \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox
   \\    /   O peration     |
    \\  /    A nd           | Copyright held by original author
     \\/     M anipulation  |
-------------------------------------------------------------------------------
License
    This file is part of OpenFOAM.

    OpenFOAM is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
    Free Software Foundation; either version 2 of the License, or (at your
    option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM; if not, write to the Free Software Foundation,
    Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

Class
    Foam::collidingSolidParticleCloud

Description
    A Cloud of solid particles

SourceFiles
    collidingSolidParticleCloudI.H
    collidingSolidParticleCloud.C
    collidingSolidParticleCloudIO.C

\*---------------------------------------------------------------------------*/

#ifndef collidingSolidParticleCloud_H
#define collidingSolidParticleCloud_H

#include "Cloud.H"
#include "collidingSolidParticle.H"
#include "IOdictionary.H"

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

namespace Foam
{

// Forward declaration of classes
class fvMesh;

/*---------------------------------------------------------------------------*\
                           Class collidingSolidParticleCloud Declaration
\*---------------------------------------------------------------------------*/

class collidingSolidParticleCloud
:
    public Cloud<collidingSolidParticle>
{
    // Private data

        const fvMesh& mesh_;

        IOdictionary particleProperties_;

        scalar rhop_;
        scalar e_;
        scalar mu_;
bool collision_;
vector U0_;

    // Private Member Functions

        //- Disallow default bitwise copy construct
        collidingSolidParticleCloud(const collidingSolidParticleCloud&);

        //- Disallow default bitwise assignment
        void operator=(const collidingSolidParticleCloud&);


public:

    // Constructors

        //- Construct given mesh
        collidingSolidParticleCloud
        (
            const fvMesh&,
            const word& cloudName = "defaultCloud"
        );


    // Member Functions

        // Access

            inline const fvMesh& mesh() const;

            inline scalar rhop() const;
            inline scalar e() const;
            inline scalar mu() const;
	    void checkCell();
bool collision(){return collision_;};
inline vector U0(){return U0_;};


        // Edit

            //- Move the particles under the influence of the given
            //  gravitational acceleration
            void move(const dimensionedVector& g);


        // I-O

            //- Write fields
            void writeFields() const;
};


// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

} // End namespace Foam

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

#include "collidingSolidParticleCloudI.H"

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

#endif

// ************************************************************************* //
